MitigationPercent = {}
MitigationPercent.name = "MitigationPercent"
MitigationPercent.version = "1.52"

local blockMitigation	= 0
local passiveMitigation	= 0
local defaultVarsVer	= 2
local defaults = {
	displayPanel		= false, 
	showInCharScreen	= true, 
	panelColorR			= 255, 
	panelColorG			= 255, 
	panelColorB			= 255, 
	textSize			= 18, 
	boldText			= false, 
	hideMode			= false,
	useBlock			= false, 
	usePassives			= false,	
	showPen				= true,
	showOther			= false,
	offset 				= {x = 920, y = 450},
	
	labelShowRes		= true,
	labelShowPwr		= false,
	labelShowCrit		= false,
	labelShowPen		= false,
}

--	CALC/FUNCTIONS				>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8======D
local function ChampionToVet(champ) --champion points to vet level
	return math.min(math.max(math.floor(champ/10), 1), 16)
end

local function getCurrentPlayerLevel()
	if IsUnitBattleLeveled("player") and IsUnitChampionBattleLeveled("player") then return GetUnitBattleLevel("player") + ChampionToVet(GetUnitChampionBattleLevel("player")) end
	if IsUnitBattleLeveled("player") then return GetUnitBattleLevel("player") end
	return GetUnitLevel("player") + ChampionToVet(GetUnitChampionPoints("player"))
end
local function getMitigation(resist)
	local percent = (resist-100) / (getCurrentPlayerLevel()*10)
	if percent < 0 then percent = 0 end
	if percent > 50 then percent = 50 end	
	return percent
end
local function getMitigationCritical(resist)
	local percent = resist / getCurrentPlayerLevel()
	if percent < 0 then percent = 0 end
	return percent
end
local function getPenetration(penetration, penType)
	local macePen = function(watDo)
		if penType == "phys" then
			local maceCount = 0
			local isTwoHand = false								
			local ActiveWeaponPair, _ = GetActiveWeaponPairInfo()
			if ActiveWeaponPair == 1 then				
				local wt = GetItemWeaponType(0, 4)				
				if wt==6 then 
					maceCount = maceCount+1
					isTwoHand = true
				else
					local wt2 = GetItemWeaponType(0, 5)
					if wt==2 and wt2~=0 and wt2~=14 then maceCount = maceCount+1 end
					if wt2==2 and wt~=0 then maceCount = maceCount+1 end
				end				
			end
			if ActiveWeaponPair == 2 then
				local wt = GetItemWeaponType(0, 20)				
				if wt==6 then 
					maceCount = maceCount+1
					isTwoHand = true
				else
					local wt2 = GetItemWeaponType(0, 21)
					if wt==2 and wt2~=0 and wt2~=14 then maceCount = maceCount+1 end
					if wt2==2 and wt~=0 then maceCount = maceCount+1 end
				end	
			end	
			if maceCount > 0 then
				local currentUpgradeLevel, maxUpgradeLevel = GetSkillAbilityUpgradeInfo( 2, 1, 7 )
				local twoHandPassiveRank = currentUpgradeLevel
				local currentUpgradeLevel, maxUpgradeLevel = GetSkillAbilityUpgradeInfo( 2, 3, 10 )
				local oneHandPassiveRank = currentUpgradeLevel
				if watDo == 0 then
					if isTwoHand then 
						return math.floor(twoHandPassiveRank*0.1 * GetPlayerStat(STAT_PHYSICAL_RESIST))
					else 
						return math.floor(oneHandPassiveRank*0.05 * maceCount * GetPlayerStat(STAT_PHYSICAL_RESIST))
					end
				end
				if watDo == 1 then
					if isTwoHand then 
						return twoHandPassiveRank*10				
					else 
						return oneHandPassiveRank*5 * maceCount 
					end
				end
			end
		end
		return 0
	end	
	
	local flatPen = penetration - macePen(0)
	local percentPenIgnored = macePen(1)
	
	if percentPenIgnored == 0 then
		return ("%03.00f"):format(flatPen) 
	else
		return ("%d%% + %03.00f"):format(percentPenIgnored, flatPen)
	end	
end
local function getPassives()	  
	blockMitigation = 0
	passiveMitigation = 0
	
	d("-------------------------------------")	  
	--passives
	if GetUnitRace("player") == "Nord" then
		local currentUpgradeLevel, maxUpgradeLevel = GetSkillAbilityUpgradeInfo( 7, 1, 4 )
		passiveMitigation = currentUpgradeLevel*2
		
		d("Nord found with passive rank " .. currentUpgradeLevel .."/3, added " .. currentUpgradeLevel*2 .."%")
	else 
		d("not nord")
	end
	  
	d("Passive mitigation: " .. passiveMitigation .."%")
	--blocking passives
	--dk iron skin
	if GetUnitClass("player") == "Dragonknight" then
		local currentUpgradeLevel, maxUpgradeLevel = GetSkillAbilityUpgradeInfo( 1, 2, 7 )
		blockMitigation = blockMitigation - currentUpgradeLevel * 5
		
		d("Class DK found with Iron Skin rank " .. currentUpgradeLevel .."/2, added " .. currentUpgradeLevel*5 .."%")
	else
		d("not DK")
	end
	  
	--swordnboard
	local currentUpgradeLevel, maxUpgradeLevel = GetSkillAbilityUpgradeInfo( 2, 2, 7 )
	blockMitigation = blockMitigation + currentUpgradeLevel * 10
	
	d("Sword and Board rank " .. currentUpgradeLevel .."/2, added " .. currentUpgradeLevel*10 .."% block increase")
	--defpost (check for skill in bar?)
	d("Defensive Posture NYI")
	d(blockMitigation .. "% overall block increase, resulting in")		
	--final blockMitigation
	blockMitigation = blockMitigation/2 + 50
	
	d(blockMitigation .. "% mitigation from blocking.")
	d("Use passives:" .. tostring(MitigationPercent.savedVariables.usePassives))
	d("Use blocking:" .. tostring(MitigationPercent.savedVariables.useBlock))
end


--	STATSCREENS					>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8======D
function showInStatScreens()
	--Character screen
	local selectSelf = {}
	local OrigAddStatRow = ZO_Stats.AddStatRow
	function ZO_Stats:AddStatRow(statType1, statType2)
		selectSelf = (select(1,self, statType1, statType2))
		OrigAddStatRow(self, statType1, statType2)
	
		if statType1 == STAT_SPELL_RESIST then
			local OrigSetText = ZO_StatsPanelPaneScrollChildStatsRow6Stat1Value.SetText
			ZO_StatsPanelPaneScrollChildStatsRow6Stat1Value.SetText = function(ctrl, text)
				local resist = tonumber(text)     
				OrigSetText(ctrl, ("%d (%03.01f%%)"):format(resist, getMitigation(resist)))
			end   
			local OrigSetText = ZO_StatsPanelPaneScrollChildStatsRow6Stat2Value.SetText
			ZO_StatsPanelPaneScrollChildStatsRow6Stat2Value.SetText = function(ctrl, text)
				local resist = tonumber(text)     
				OrigSetText(ctrl, ("%d (%03.01f%%)"):format(resist, getMitigation(resist)))
			end
		end
				
		if statType1 == STAT_CRITICAL_RESISTANCE then
			local OrigSetText = ZO_StatsPanelPaneScrollChildStatsRow7Stat1Value.SetText
			ZO_StatsPanelPaneScrollChildStatsRow7Stat1Value.SetText = function(ctrl, text)
				local resist = tonumber(text)     
				OrigSetText(ctrl, ("%d (%03.01f%%)"):format(resist, getMitigationCritical(resist)))
			end   
		end
				
		if statType1 == STAT_SPELL_PENETRATION then
			local OrigSetText = ZO_StatsPanelPaneScrollChildStatsRow8Stat1Value.SetText
			ZO_StatsPanelPaneScrollChildStatsRow8Stat1Value.SetText = function(ctrl, text)
				local penetration = tonumber(text)
				OrigSetText(ctrl, getPenetration(penetration, "spell"))			
			end
			local OrigSetText = ZO_StatsPanelPaneScrollChildStatsRow8Stat2Value.SetText
			ZO_StatsPanelPaneScrollChildStatsRow8Stat2Value.SetText = function(ctrl, text)
				local penetration = tonumber(text)     
				OrigSetText(ctrl, getPenetration(penetration, "phys"))	
			end
		end		
	end
	
	local OrigCreateAttributesSection = ZO_Stats.CreateAttributesSection
	function ZO_Stats.CreateAttributesSection(...)
		local attSec = OrigCreateAttributesSection(...)
		if MitigationPercent.savedVariables.showPen then
			ZO_Stats.SetNextControlPadding(selectSelf,15)
			ZO_Stats.AddStatRow(selectSelf, STAT_SPELL_PENETRATION, STAT_PHYSICAL_PENETRATION)
		end
		return attSec
		--ZO_Stats.AddStatRow(self, STAT_DAMAGE_RESIST_COLD, STAT_DAMAGE_RESIST_DISEASE)
		--ZO_Stats.AddStatRow(self, STAT_DAMAGE_RESIST_DROWN, STAT_DAMAGE_RESIST_EARTH)
		--ZO_Stats.AddStatRow(self, STAT_DAMAGE_RESIST_FIRE, STAT_DAMAGE_RESIST_GENERIC)
		--ZO_Stats.AddStatRow(self, STAT_DAMAGE_RESIST_MAGIC, STAT_DAMAGE_RESIST_OBLIVION)
		--ZO_Stats.AddStatRow(self, STAT_DAMAGE_RESIST_PHYSICAL, STAT_DAMAGE_RESIST_POISON)
		--ZO_Stats.AddStatRow(self, STAT_DAMAGE_RESIST_SHOCK)
	end

	--Inventory screen
	local invetoryStats = {{STAT_SPELL_PENETRATION, STAT_PHYSICAL_PENETRATION,},}
	local lastInvStatControl = ZO_CharacterWindowStatsScrollScrollChildStatEntry24
	if MitigationPercent.savedVariables.showPen then
		local parentControl = ZO_CharacterWindowStats:GetNamedChild("ScrollScrollChild")
		local lastControl = lastInvStatControl
		local nextPaddingY = 20
		for _, statGroup in ipairs(invetoryStats) do
			for _, stat in ipairs(statGroup) do
				local statControl = CreateControlFromVirtual("$(parent)StatEntry", parentControl, "ZO_StatsEntry", stat)
				local relativeAnchorSide = (lastControl == nil) and TOP or BOTTOM
				statControl:SetAnchor(TOP, lastControl, relativeAnchorSide, 0, nextPaddingY)

				local statEntry = ZO_StatEntry_Keyboard:New(statControl, stat)
				statEntry.tooltipAnchorSide = LEFT
				lastControl = statControl
				nextPaddingY = 5
			end
			nextPaddingY = 20
		end

		local OrigSetText = ZO_CharacterWindowStatsScrollScrollChildStatEntry33Value.SetText
		function ZO_CharacterWindowStatsScrollScrollChildStatEntry33Value.SetText(ctrl, text)
			local penetration = tonumber(text)     
			OrigSetText(ctrl, getPenetration(penetration, "phys"))
		end
		local OrigSetText = ZO_CharacterWindowStatsScrollScrollChildStatEntry34Value.SetText
		function ZO_CharacterWindowStatsScrollScrollChildStatEntry34Value.SetText(ctrl, text)
			local penetration = tonumber(text)     
			OrigSetText(ctrl, getPenetration(penetration, "spell"))	
		end
	end
	
	local OrigSetText = ZO_CharacterWindowStatsScrollScrollChildStatEntry13Value.SetText
	function ZO_CharacterWindowStatsScrollScrollChildStatEntry13Value.SetText(ctrl, text)
		local resist = tonumber(text)
		OrigSetText(ctrl, ("%d (%03.01f%%)"):format(resist, getMitigation(resist)))
	end
	local OrigSetText = ZO_CharacterWindowStatsScrollScrollChildStatEntry22Value.SetText
	function ZO_CharacterWindowStatsScrollScrollChildStatEntry22Value.SetText(ctrl, text)
		local resist = tonumber(text)
		OrigSetText(ctrl, ("%d (%03.01f%%)"):format(resist, getMitigation(resist)))
	end
	
	local OrigSetText = ZO_CharacterWindowStatsScrollScrollChildStatEntry24Value.SetText
	function ZO_CharacterWindowStatsScrollScrollChildStatEntry24Value.SetText(ctrl, text)
		local resist = tonumber(text)
		OrigSetText(ctrl, ("%d (%03.01f%%)"):format(resist, getMitigationCritical(resist)))
	end

	--Gamepad (this is broken atm)
	-- if MitigationPercent.savedVariables.showPen then
		-- local gamepadStatsLC = ZO_GamepadStatsTopLevelRightPaneInfoPanelAttributesPanelLeftColumn
		-- local gamepadStatsRC = ZO_GamepadStatsTopLevelRightPaneInfoPanelAttributesPanelRightColumn
		-- local spellPenHeader = CreateControlFromVirtual("SpellPenetrationHeader", gamepadStatsLC, "ZO_GamepadStatsHeaderTemplate")
		-- spellPenHeader:ClearAnchors()
		-- spellPenHeader:SetAnchor(TOPLEFT, ZO_GamepadStatsTopLevelRightPaneInfoPanelAttributesPanelLeftColumnRidingSpeedHeader, BOTTOMLEFT, 0, 90)
		-- local spellPenData = CreateControlFromVirtual("SpellPenetration", gamepadStatsLC, "ZO_GamepadStatsDataTemplate")
		-- spellPenData:ClearAnchors()
		-- spellPenData:SetAnchor(TOPRIGHT, ZO_GamepadStatsTopLevelRightPaneInfoPanelAttributesPanelLeftColumnRidingSpeed, BOTTOMRIGHT, 0, 90)
		-- local physPenHeader = CreateControlFromVirtual("PhysicalPenetrationHeader", gamepadStatsRC, "ZO_GamepadStatsHeaderTemplate")
		-- physPenHeader:ClearAnchors()
		-- physPenHeader:SetAnchor(TOPLEFT, ZO_GamepadStatsTopLevelRightPaneInfoPanelAttributesPanelRightColumnRidingTrainingHeader, BOTTOMLEFT, 0, 50)
		-- local physPenData = CreateControlFromVirtual("PhysicalPenetration", gamepadStatsRC, "ZO_GamepadStatsDataTemplate")
		-- physPenData:ClearAnchors()
		-- physPenData:SetAnchor(TOPRIGHT, ZO_GamepadStatsTopLevelRightPaneInfoPanelAttributesPanelRightColumnRidingTrainingTimer, BOTTOMRIGHT, 0, 50)

		-- local OrigInitializeCharacterStats = ZO_GamepadStats.InitializeCharacterStats
		-- function ZO_GamepadStats:InitializeCharacterStats()
			-- OrigInitializeCharacterStats(self)
			-- self.spellPenetrationHeader = spellPenHeader
			-- self.physicalPenetrationHeader = physPenHeader
			-- table.insert(self.headers, { label = self.spellPenetrationHeader, stat = STAT_SPELL_PENETRATION })
			-- table.insert(self.headers, { label = self.physicalPenetrationHeader, stat = STAT_PHYSICAL_PENETRATION })					
			-- self.physicalPenetration = physPenData
			-- self.spellPenetration = spellPenData
			-- table.insert(self.statValues, { label = self.spellPenetration, stat = STAT_SPELL_PENETRATION })
			-- table.insert(self.statValues, { label = self.physicalPenetration, stat = STAT_PHYSICAL_PENETRATION })
		-- end

		-- local OrigSetText = spellPenData.SetText
		-- function spellPenData.SetText(ctrl, text)
			-- local penetration = tonumber(text)
			-- OrigSetText(ctrl, getPenetration(penetration, "spell"))
		-- end
		-- local OrigSetText = physPenData.SetText
		-- function physPenData.SetText(ctrl, text)
			-- local penetration = tonumber(text)
			-- OrigSetText(ctrl, getPenetration(penetration, "phys"))
		-- end
	-- end
	
	-- local OrigSetText = ZO_GamepadStatsTopLevelRightPaneInfoPanelAttributesPanelLeftColumnSpellResistance.SetText
	-- function ZO_GamepadStatsTopLevelRightPaneInfoPanelAttributesPanelLeftColumnSpellResistance.SetText(ctrl, text)
		-- local resist = tonumber(text)
		-- OrigSetText(ctrl, ("%d (%03.01f%%)"):format(resist, getMitigation(resist)))
	-- end
	-- local OrigSetText = ZO_GamepadStatsTopLevelRightPaneInfoPanelAttributesPanelRightColumnArmor.SetText
	-- function ZO_GamepadStatsTopLevelRightPaneInfoPanelAttributesPanelRightColumnArmor.SetText(ctrl, text)
		-- local resist = tonumber(text)
		-- OrigSetText(ctrl, ("%d (%03.01f%%)"):format(resist, getMitigation(resist)))
	-- end
	
	-- local OrigSetText = ZO_GamepadStatsTopLevelRightPaneInfoPanelAttributesPanelLeftColumnCritResistance.SetText
	-- function ZO_GamepadStatsTopLevelRightPaneInfoPanelAttributesPanelLeftColumnCritResistance.SetText(ctrl, text)
		-- local resist = tonumber(text)
		-- OrigSetText(ctrl, ("%d (%03.01f%%)"):format(resist, getMitigationCritical(resist)))
	-- end
end


--	PANEL						>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8======D
function MitigationPercent.OnStatsUpdated(event, unitTag)
	MitigationPercent:UpdatePanel()
end
function MitigationPercent:UpdatePanel()
	local mitigationPhysical = getMitigation(GetPlayerStat(STAT_PHYSICAL_RESIST))
	local mitigationSpell = getMitigation(GetPlayerStat(STAT_SPELL_RESIST))	
	-- TODO: is passive (nord) before block or after, or with or after resistances, hardcap?? pls help :(
	--Add nord passive to mitigation
	if self.savedVariables.usePassives then
		mitigationSpell = mitigationSpell + passiveMitigation
		mitigationPhysical = mitigationPhysical + passiveMitigation
	end
	--Add blockMitigation to mitigation
	if self.savedVariables.useBlock and IsBlockActive() then
		local blockMitigationDec = (100 - blockMitigation) / 100
		mitigationSpell = 100 - (100 - mitigationSpell) * blockMitigationDec
		mitigationPhysical = 100 - (100 - mitigationPhysical) * blockMitigationDec
	end
	
	local panelString = ""
	if self.savedVariables.labelShowRes then panelString = ("SR: %03.01f%%  PR: %03.01f%%"):format(mitigationSpell, mitigationPhysical) end
	if self.savedVariables.labelShowPwr then panelString = panelString .. ("\nSP: %d  WD: %d"):format(GetPlayerStat(STAT_SPELL_POWER, 0, 0), GetPlayerStat(STAT_POWER, 0, 0)) end
	if self.savedVariables.labelShowCrit then panelString = panelString .. ("\nSC: %03.01f%%  WC: %03.01f%%"):format(GetCriticalStrikeChance(GetPlayerStat(STAT_SPELL_CRITICAL, 0, 0), true), GetCriticalStrikeChance(GetPlayerStat(STAT_CRITICAL_STRIKE, 0, 0), true)) end
	if self.savedVariables.labelShowPen then panelString = panelString .. "\nSP: "..getPenetration(GetPlayerStat(STAT_SPELL_PENETRATION, 0, 0), "spell") .." PP: ".. getPenetration(GetPlayerStat(STAT_PHYSICAL_PENETRATION, 0, 0), "phys") end
	self.ctrl.label:SetText(panelString)
end
function MitigationPercent.onBuffChange(eventCode, changeType, effectSlot, effectName, unitTag, beginTime, endTime, stackCount, iconName, buffType, effectType, abilityType, statusEffectType)
  if unitTag == "player" then
    if effectName == "Brace (Generic)" then
      MitigationPercent:UpdatePanel()
    end
  end
end


--	INITIALIZE					>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8======D
function MitigationPercent.OnAddOnLoaded(event, addonName)
  if addonName == MitigationPercent.name then
	EVENT_MANAGER:UnregisterForEvent(MitigationPercent.name, EVENT_ADD_ON_LOADED)
    MitigationPercent:Initialize()
  end
end
function MitigationPercent:Initialize()	
	self:RestoreData()
	self:SetupAddonMenu()
end
function MitigationPercent:RestoreData()
	self.savedVariables = ZO_SavedVars:New("MitigationPercentSavedVariables", defaultVarsVer, nil, defaults)
	
	self:InitializeControls()
	
	self.panelFragment = ZO_HUDFadeSceneFragment:New(MitigationPercentDisplay, 200, 0)
	self.hudScene = SCENE_MANAGER:GetScene("hud")
	self.hudUiScene	= SCENE_MANAGER:GetScene("hudui")
	
	if self.savedVariables.displayPanel == true then
		MitigationPercentDisplay:SetHidden(false)
		MitigationPercentDisplay:SetMouseEnabled(true)
		MitigationPercentDisplay:SetMovable(true)
		EVENT_MANAGER:RegisterForEvent(self.name, EVENT_STATS_UPDATED, self.OnStatsUpdated)
		EVENT_MANAGER:AddFilterForEvent(self.name, EVENT_STATS_UPDATED, REGISTER_FILTER_UNIT_TAG, "player")
		self:UpdatePanel()
			
		if self.savedVariables.hideMode == true then
			MitigationPercentDisplay:SetMovable(false)
			MitigationPercentDisplay:SetMouseEnabled(false)
			self.hudScene:AddFragment(self.panelFragment)
			self.hudUiScene:AddFragment(self.panelFragment)
		end	
	end	
	
	if self.savedVariables.useBlock then
		EVENT_MANAGER:RegisterForEvent(self.name, EVENT_EFFECT_CHANGED, self.onBuffChange)
		EVENT_MANAGER:AddFilterForEvent(self.name, EVENT_EFFECT_CHANGED, REGISTER_FILTER_UNIT_TAG, "player")
	end

	if self.savedVariables.showInCharScreen then showInStatScreens() end
	
	getPassives()	
end

EVENT_MANAGER:RegisterForEvent(MitigationPercent.name, EVENT_ADD_ON_LOADED, MitigationPercent.OnAddOnLoaded)
SLASH_COMMANDS["/mpupdate"] = getPassives